use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/WWW/Mechanize.pm',
    'lib/WWW/Mechanize/Cookbook.pod',
    'lib/WWW/Mechanize/Examples.pod',
    'lib/WWW/Mechanize/FAQ.pod',
    'lib/WWW/Mechanize/Image.pm',
    'lib/WWW/Mechanize/Link.pm',
    'script/mech-dump',
    't/00-load.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/TestServer.pm',
    't/add_header.t',
    't/aliases.t',
    't/area_link.html',
    't/area_link.t',
    't/autocheck.t',
    't/bad-request.t',
    't/clone.t',
    't/content.t',
    't/cookies.t',
    't/credentials-api.t',
    't/credentials.t',
    't/die.t',
    't/dump.t',
    't/field.html',
    't/field.t',
    't/file_upload.html',
    't/file_upload.t',
    't/find_frame.html',
    't/find_frame.t',
    't/find_image.t',
    't/find_inputs.html',
    't/find_inputs.t',
    't/find_link-warnings.t',
    't/find_link.html',
    't/find_link.t',
    't/find_link_id.html',
    't/find_link_id.t',
    't/find_link_xhtml.html',
    't/find_link_xhtml.t',
    't/form-parsing.t',
    't/form_133_regression.html',
    't/form_with_fields.html',
    't/form_with_fields.t',
    't/form_with_fields_passthrough_params.t',
    't/form_with_fields_verbose.html',
    't/frames.html',
    't/frames.t',
    't/google.html',
    't/history.t',
    't/history_1.html',
    't/history_2.html',
    't/history_3.html',
    't/html_file.txt',
    't/image-new.t',
    't/image-parse.css',
    't/image-parse.html',
    't/image-parse.t',
    't/link-base.t',
    't/link-relative.t',
    't/link.t',
    't/local/LocalServer.pm',
    't/local/back.t',
    't/local/click.t',
    't/local/click_button.t',
    't/local/content.t',
    't/local/encoding.t',
    't/local/failure.t',
    't/local/follow.t',
    't/local/form.t',
    't/local/get.t',
    't/local/head.t',
    't/local/log-server',
    't/local/nonascii.html',
    't/local/nonascii.t',
    't/local/overload.t',
    't/local/page_stack.t',
    't/local/post.t',
    't/local/referer-server',
    't/local/referer.t',
    't/local/reload.t',
    't/local/select_multiple.t',
    't/local/submit.t',
    't/mech-dump/file_not_found.t',
    't/mech-dump/mech-dump.t',
    't/new.t',
    't/refresh.html',
    't/regex-error.t',
    't/save_content.html',
    't/save_content.t',
    't/select.html',
    't/select.t',
    't/submit_form.t',
    't/taint.t',
    't/tick.html',
    't/tick.t',
    't/untaint.t',
    't/upload.html',
    't/upload.t',
    't/uri.t',
    't/warn.t',
    't/warnings.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
